#!/usr/bin/perl -w

# submit.cgi: submit applications for Faculty positions
#
# Copyright (C) 2000-2003 Filippo Menczer and the University of Iowa
# Copyright (C) 2003-2004 Filippo Menczer and Indiana University 
#
#    This file is part of the Recruit system
#
#    Recruit is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Recruit is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Recruit; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# CONVENTIONS: 
# - nextID is stored in file 'recruit-ID'
# - applicant's info is stored in database ./admin/$ID/$ID.DB
# - applicant's CV is stored in file $path/$ID/$ID-cvfile.pfd
# - applicant's statement is stored in file $path/$ID/$ID-stmtfile.pfd
# - committee info is stored in database './admin/committee.DB'

$|=1;
use strict 'vars';
use CGI::Carp qw(fatalsToBrowser);
use CGI qw(:standard);
use Fcntl qw(:flock O_RDWR O_CREAT);
use DB_File;

# PARAMETERS OF THE SEARCH
use lib qw(.);
use params ();
use vars qw(%global);
*global = \%params::global;
my $dept = $global{dept};
my $path = $global{path};
my $adminmail = $global{adminmail};
my $startid = $global{startid};
my $areas = $global{areas};
my $mailflag = $global{mailflag};

# OTHER GLOBAL
my $format = "Merge multiple documents if necessary - only one documents can be uploaded in this field. <b>Only PDF format accepted!</b>";

# PARAMS ORDERED FOR CONVENIENCE 
# 'cvfile' and 'stmtfile' skipped b/c not in DB
# 'areas' skipped b/c must be preprocessed)
# The date is computed at runtime
# The 'new' flag is set to 1 and changed when an ack letter is generated
my @parameters = qw(
	 fname lname 
	 url email fax phone 
	 address affil school dept 
	 phdmonth phdyear 
	 ref1title ref1fname ref1lname ref1affil ref1email ref1fax ref1phone ref1note
	 ref2title ref2fname ref2lname ref2affil ref2email ref2fax ref2phone ref2note
	 ref3title ref3fname ref3lname ref3affil ref3email ref3fax ref3phone ref3note
	 ref4title ref4fname ref4lname ref4affil ref4email ref4fax ref4phone ref4note
	 ref5title ref5fname ref5lname ref5affil ref5email ref5fax ref5phone ref5note
	 ref6title ref6fname ref6lname ref6affil ref6email ref6fax ref6phone ref6note
);

# get next ID from file if it exists, else 0, and lock access to file
sub nextID {
	 my $IDfile = 'recruit-ID';
	 sysopen(ID, $IDfile, O_RDWR|O_CREAT) || die "cannot open $IDfile: $!";
	 flock(ID, LOCK_EX) || die "cannot lock $IDfile: $!"; 
	 my $id = <ID> || $startid;
	 chomp($id);
	 seek(ID,0,0) || die "cannot rewind $IDfile: $!";
	 truncate(ID,0) || die "cannot truncate $IDfile: $!";
	 print(ID $id+1, "\n") || die "cannot write to $IDfile: $!";
	 close(ID) || die "cannot close $IDfile: $!";
	 return $id;
}

# return true if any candidate areas match reviewer profile
sub matches {
	my($profile_ref, @cand_areas) = @_;	
	my %profile;	
	@profile{@$profile_ref} = undef;	
	foreach my $area (@cand_areas) {
		return 1 if exists $profile{$area};
	}
	return 0;
}

# send email to notify committee members of new application
sub mailnotify ($$@) {
	my ($subj, $body, @addr) = @_;
	require Mail::Mailer;
	Mail::Mailer->import();
	eval { # if mail delivery fails the script won't die
		my $mailer = Mail::Mailer->new;
		foreach my $to (@addr) {
			$mailer->open({To => $to, Subject => "New Recruit App ($subj)"});
			print $mailer $body;
			$mailer->close() or die "Cannot close mailer: $!\n";	
		}
	};
}

# MAIN
print header, start_html(-title=>"$dept Recruit", -BGCOLOR=>'ffffff'), 
	 h1("Application for faculty positions in $dept");
	 
if (!param()) { # first time through, present clean form

    print start_multipart_form();
    
    print h2("1. Personal information"); 
    
	print "All fields are required unless otherwise stated.";
    print "<table cellpadding=10 border=5><td><table>\n";
	 print Tr(td("First Name:"), td( textfield('fname','',50)));
	 print Tr(td("Last Name:"), td( textfield('lname','',50)));
	 print Tr(td("Email:"), td( textfield('email','',50)));
	 print Tr(td("Homepage (optional):"), td( textfield('url','http://',50)));
	 print Tr(td("Fax (optional):"), td( textfield('fax','',15)));
	 print Tr(td("Phone (optional):"), td( textfield('phone','',15)));
	 print Tr(td("Address:"), td( textarea('address','',5,50)));
	 print Tr(td("Current Affiliation<br>(if different from address):"), td( textarea('affil','',5,50)));
    print "</table></td></table>\n";
	 
    print h2("2. Academic information"); 
    
    print "<table cellpadding=10 border=5><td><table>\n";
	 print Tr(td("PhD Institution:"), td( textfield('school','',50)));
	 print Tr(td("PhD Department:"), td( textfield('dept','',50)));
	 print Tr(td("PhD received/anticipated:"), 
		 td("Month:" . textfield('phdmonth','', 2, 2) . "Year:" . textfield('phdyear','', 5, 4)));
    print Tr(td("Research area(s):"), td( checkbox_group('areas', $areas, [], 'true'))); 
    print "</table></td></table>\n";
    
    print h2("3. References"); 
    
    print "<table cellpadding=10 border=5>\n";
	 print Tr(th("Reference 1"), td("<table>" .
		  Tr(td("Title: "), td(textfield('ref1title','',4))) . 
		  Tr(td("First Name: "), td(textfield('ref1fname','',50))) . 
		  Tr(td("Last Name: "), td(textfield('ref1lname','',50))) . 
		  Tr(td("Affiliation: "), td(textfield('ref1affil','',50))) . 
		  Tr(td("Email: "), td(textfield('ref1email','',50))) . 
		  Tr(td("Phone: "), td(textfield('ref1phone','',15))) .
		  Tr(td("Fax: "), td(textfield('ref1fax','',15))) .
		  Tr(td("Notes: "), td(textarea('ref1note','',3,50))) .
		  "</table>"
		  ));
    print "</table>\n", p;
    print "<table cellpadding=10 border=5>\n";
	 print Tr(th("Reference 2"), td("<table>" .
		  Tr(td("Title: "), td(textfield('ref2title','',4))) . 
		  Tr(td("First Name: "), td(textfield('ref2fname','',50))) . 
		  Tr(td("Last Name: "), td(textfield('ref2lname','',50))) . 
		  Tr(td("Affiliation: "), td(textfield('ref2affil','',50))) . 
		  Tr(td("Email: "), td(textfield('ref2email','',50))) . 
		  Tr(td("Phone: "), td(textfield('ref2phone','',15))) .
		  Tr(td("Fax: "), td(textfield('ref2fax','',15))) .
		  Tr(td("Notes: "), td(textarea('ref2note','',3,50))) .
		  "</table>"
		  ));
    print "</table>\n", p;
    print "<table cellpadding=10 border=5>\n";
	 print Tr(th("Reference 3"), td("<table>" .
		  Tr(td("Title: "), td(textfield('ref3title','',4))) . 
		  Tr(td("First Name: "), td(textfield('ref3fname','',50))) . 
		  Tr(td("Last Name: "), td(textfield('ref3lname','',50))) . 
		  Tr(td("Affiliation: "), td(textfield('ref3affil','',50))) . 
		  Tr(td("Email: "), td(textfield('ref3email','',50))) . 
		  Tr(td("Phone: "), td(textfield('ref3phone','',15))) .
		  Tr(td("Fax: "), td(textfield('ref3fax','',15))) .
		  Tr(td("Notes: "), td(textarea('ref3note','',3,50))) .
		  "</table>"
		  ));
    print "</table>\n", p;
	 print "<table cellpadding=10 border=5>\n";
	 print Tr(th("Reference 4"), td("<table>" .
		  Tr(td("Title: "), td(textfield('ref4title','',4))) . 
		  Tr(td("First Name: "), td(textfield('ref4fname','',50))) . 
		  Tr(td("Last Name: "), td(textfield('ref4lname','',50))) . 
		  Tr(td("Affiliation: "), td(textfield('ref4affil','',50))) . 
		  Tr(td("Email: "), td(textfield('ref4email','',50))) . 
		  Tr(td("Phone: "), td(textfield('ref4phone','',15))) .
		  Tr(td("Fax: "), td(textfield('ref4fax','',15))) .
		  Tr(td("Notes: "), td(textarea('ref4note','',3,50))) .
		  "</table>"
		  ));
	 print "</table>\n", p;
	 print "<table cellpadding=10 border=5>\n";
	 print Tr(th("Reference 5"), td("<table>" .
		  Tr(td("Title: "), td(textfield('ref5title','',4))) . 
		  Tr(td("First Name: "), td(textfield('ref5fname','',50))) . 
		  Tr(td("Last Name: "), td(textfield('ref5lname','',50))) . 
		  Tr(td("Affiliation: "), td(textfield('ref5affil','',50))) . 
		  Tr(td("Email: "), td(textfield('ref5email','',50))) . 
		  Tr(td("Phone: "), td(textfield('ref5phone','',15))) .
		  Tr(td("Fax: "), td(textfield('ref5fax','',15))) .
		  Tr(td("Notes: "), td(textarea('ref5note','',3,50))) .
		  "</table>"
		  ));
	 print "</table>\n", p;
	 print "<table cellpadding=10 border=5>\n";
	 print Tr(th("Reference 6"), td("<table>" .
		  Tr(td("Title: "), td(textfield('ref6title','',4))) . 
		  Tr(td("First Name: "), td(textfield('ref6fname','',50))) . 
		  Tr(td("Last Name: "), td(textfield('ref6lname','',50))) . 
		  Tr(td("Affiliation: "), td(textfield('ref6affil','',50))) . 
		  Tr(td("Email: "), td(textfield('ref6email','',50))) . 
		  Tr(td("Phone: "), td(textfield('ref6phone','',15))) .
		  Tr(td("Fax: "), td(textfield('ref6fax','',15))) .
		  Tr(td("Notes: "), td(textarea('ref6note','',3,50))) .
		  "</table>"
		  ));
	 print "</table>\n";
		  
    print h2("4. Documents"); 
    
    print "<table cellpadding=10 border=5><td><table>\n";
	 print Tr(td("Upload <b>curriculum vitae including list of publications</b>. $format"), 
		 td(filefield('cvfile','',40)));
	 print "</table></td></table>\n", p;
	 print "<table cellpadding=10 border=5><td><table>\n";
	 print Tr(td("Upload <b>statement of research interests and teaching philosophy</b>. $format"), 
		 td(filefield('stmtfile','',40)));
    print "</table></td></table>\n";
	 
    print h2("5. Done"); 
    
	 print p(submit("Submit Application!"), reset("Clear Form"));
    print end_form();
	 
} else { # form already filled out
	 
	 # If docs OK, create directory named after ID and save two renamed files there
	 # NB: upload_file param is a handle to access file content
	 die "Missing required fields -" unless (param('fname') && param('lname') && param('email') && param('address'));
	 die "Missing required documents -" unless (param('cvfile') && param('stmtfile'));
	 die "You must specify at least one area -" unless param('areas');
	 my $handle;
	 $handle = param('cvfile');
	 my $first_cv = <$handle>;
	 $handle = param('stmtfile');
	 my $first_stmt = <$handle>;
	 die "Only PDF documents accepted -" unless (($first_cv =~ /^%PDF/) && ($first_stmt =~ /^%PDF/));
	 my $ID = nextID();
	 mkdir("$path/$ID", 0777) || die "cannot create directory: $!";
	 foreach my $f ('cvfile', 'stmtfile') {
	 	 $handle = param($f);
	 	 seek($handle, 0, 0) || die "cannot rewind file: $!"; # rewind after checking first line for format
		 my $filename = "$path/$ID/$ID-$f.pdf";
		 open(F, ">$filename") || die "cannot create $filename: $!";
		 my @file = <$handle>;
		 print F @file;
		 close F;
	 }
	 
	 # save all fields in database file in candidate ID's (local) directory
	 # NB: areas must be preprocessed to join array into string of keywords
	 #     and the application date is computed on the fly
	 #     and the 'new' flag is set to true (until changed by addletter.cgi)
	 #     and address and affiliation must be formatted for line breaks
	 #     and url must be deleted if empty
	 # ALSO write email notification body
	 mkdir("admin/$ID", 0750) or die "cannot create directory: $!";
	 my %candidate;
	 tie(%candidate, 'DB_File', "admin/$ID/$ID.DB") || die "Cannot create candidate $ID DB_File: $!";
	 my $mailbody = '';
	 $candidate{'areas'} = join(', ', param('areas'));
	 my $mailsubject = $candidate{'areas'};
	 $mailbody .= "areas: $candidate{'areas'}\n";
	 foreach my $p (@parameters) {
	 	$candidate{$p} = param($p);
	 	$mailbody .= "$p: $candidate{$p}\n";
	 }
	 my ($day, $month, $year) = (localtime)[3,4,5];
	 $month++; $year+=1900;
	 $candidate{'date'} = "$month/$day/$year";
	 $candidate{'new'} = "New";
	 $candidate{'address'} =~ s/\n/<br>/g;
	 $candidate{'affil'} =~ s/\n/<br>/g;
	 $candidate{'url'} = '' if ($candidate{'url'} eq "http://");
	 untie %candidate;
	 
	 # ACK APPLICANT AND NOTIFY COMMITTEE
	 print p("Thank you for your application! You will receive an acknowledgment via snail mail.");
	 print p("For questions contact the $dept department at <a href=\"mailto:$adminmail\">$adminmail</a>, referring to application number: $ID.");
	 if ($mailflag) {
	 	 my %comm;
		 my @addresses;
		 tie(%comm, 'DB_File', "admin/committee.DB") || die "Cannot open committee DB_File: $!";
		 while (my($k, $v) = each %comm) {
		 	my ($mail, $flag, @profile) = split(/$;/, $v);
		 	push(@addresses, $mail) if ($flag eq '1' && matches(\@profile, param('areas')));
		 }
		 untie %comm;
		 mailnotify($mailsubject, $mailbody, @addresses);
	 } 
}
print hr, a({href=>"http://www.informatics.indiana.edu/fil/Recruit/", target=>"_new"}, 'Recruit'), " &copy; 2000-2004 ", a({href=>"http://www.informatics.indiana.edu/fil/", target=>"_new"}, 'Filippo Menczer'); 
print end_html;

